#include "Confidence.h"

Confidence::Confidence(std::shared_ptr<int> lastKey, std::shared_ptr<UI> ui, unsigned validityLength, float similarity) {
	queueLength = validityLength;
	this->lastKey = lastKey;
	this->ui = ui;
	this->similarity = similarity;
}

bool Confidence::check(std::vector<cv::Rect> bounds) {
	bool consistent;
	confident.clear();

	if (*lastKey == 98) {
		drawBounds = !drawBounds;
	}

	for (unsigned rectangleID = 0; rectangleID < bounds.size(); rectangleID++) {
		consistent = true;

		if (confidenceQueue.size() < queueLength) {
			consistent = false;
		}
		cv::Rect current = bounds[rectangleID];

		for (unsigned frameID = 0; frameID < confidenceQueue.size(); frameID++) {
			if (confidenceQueue[frameID].size() == 0) {
				consistent = false;
				break;
			}
			for (unsigned frameRectID = 0; frameRectID < confidenceQueue[frameID].size(); frameRectID++) {
				cv::Rect previous = confidenceQueue[frameID][frameRectID];
				if ((current & previous).area() < similarity*(current.area())) {
					consistent = false;
					break;
				}
			}
		}
		if (consistent) {
			confident.push_back(bounds[rectangleID]);
		}
	}

	confidenceQueue.push_front(bounds);
	if (confidenceQueue.size() > queueLength) {
		confidenceQueue.pop_back();
	}

	if (drawBounds) {
		drawBoundsOnMat(*(ui->drawnFrame()));
		cv::putText(*(ui->drawnFrame()), "Showing confident rectangles.",
				cv::Point{5, 45}, cv::FONT_HERSHEY_PLAIN, 0.8, boundColor);
	}

	return (confident.size() > 0);
}

std::vector<cv::Rect> Confidence::rectangleBounds() const {
	return confident;
}

void Confidence::drawBoundsOnMat(cv::Mat &mat) {
	for (unsigned i = 0; i < confident.size(); i++) {
		cv::rectangle(mat, confident[i], boundColor, 1);
	}
}


Confidence::~Confidence() {
}
