#include "Ruler.h"
#include <sstream>

Ruler::Ruler(std::shared_ptr<UI> ui, std::shared_ptr<int> lastKey) {
	this->ui = ui;
	this->lastKey = lastKey;
	ui->UIListeners.push_back(this);
}

void Ruler::update() {
	focalLength = estimateFocalLength();
	distance = estimateDistance();
	inputUpdate();
	updateUI();
}

float Ruler::estimateFocalLength() {
	float fl = (ui->getWidth()/2.0) * (cos((HFOV * M_PI)/(180*2.0))/sin(HFOV*M_PI/(180*2.0)));
	return fl;
}

void Ruler::updateUI() {
	if (inputtingVal) {
		cv::putText(*(ui->drawnFrame()), ("Given width: " + std::to_string(givenWidth) + "cm"), confTextPos, cv::FONT_HERSHEY_PLAIN, 0.7, textFontColor, 2);
	} else {
		cv::putText(*(ui->drawnFrame()), ("Given width: " + std::to_string(givenWidth) + "cm"), confTextPos, cv::FONT_HERSHEY_PLAIN, 0.7, textFontColor);
	}
	cv::putText(*(ui->drawnFrame()), ("distance: " + std::to_string(distance) + "cm"), distanceTextPos, cv::FONT_HERSHEY_PLAIN, 0.7, textFontColor);
}

void Ruler::toggleInput() {
	inputtingVal = !inputtingVal;
	if (!inputtingVal) {
		numBuild.clear();
	} else {
		givenWidth = 0;
	}
}

int Ruler::estimateDistance() {
	if (detectedWidth > 0) {
		return (givenWidth*focalLength) /detectedWidth;
	} else {
		return 0;
	}
}

void Ruler::inputUpdate() {
	if (*lastKey == 13) {
		toggleInput();
		decimal = false;

	}
	if (inputtingVal) {
		if (*lastKey >= 48 && *lastKey <= 57) {
			char val = '0' + (*lastKey - 48);
			numBuild.push_back(val);
		} else if (*lastKey == 46 && decimal == false) {
			char val = '.';
			numBuild.push_back(val);
			decimal = true;
		}
		if (numBuild.size() > 0) {
			std::string numString{numBuild.begin(), numBuild.end()};
			givenWidth = std::stof(numString);
		}
	}
}

void Ruler::setDetectedWidth(int width) {
	detectedWidth = width;
}

void Ruler::componentSetup() {
	cv::createTrackbar("HFOV", UI::DEBUG_WINDOW, &HFOV , 100);
}

Ruler::~Ruler() {
	ui->UIListeners.erase(std::remove(ui->UIListeners.begin(), ui->UIListeners.end(), this));
}

